-- Setup your account at http://emoncms.org/
-- See API documentation at http://emoncms.org/input/api

-- API Key
local API_KEY = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"

-- Setup your devices here. You can use a function to calculate the power as illustrated in the sample.
-- For device logging, use: key, deviceId, serviceId, serviceVar
-- For function based logging, use: key, calculate, serviceVar
local VARIABLES = {
	{ key="Livingroom", deviceId = 12, serviceId='urn:micasaverde-com:serviceId:EnergyMetering1', serviceVar="Watts" }, -- Send device energy
	{ key="Kitchen", deviceId = 13, serviceId='urn:micasaverde-com:serviceId:EnergyMetering1', serviceVar="Watts" }, 
	{ key='KitchenSwitch', deviceId=13, serviceId="urn:upnp-org:serviceId:SwitchPower1", serviceVar="Status"}, -- Send switch status
	{ key='Computer', calculate=function() return (IsComputerPingSensorTripped() and 38 or 1) end, serviceVar="Watts" }, -- Send variable value
	{ key='Other', calculate=function() return 15 end, serviceVar="Watts" } -- Send a constant value
}

local NODE_ID = 10
local TOTAL_KEY = 'Total'

-- Upload Frequency in seconds
local updateInterval = 10

-- Log debug messages
local DEBUG = true

-- You shouldn't need to change anything below this line --


local http = require('socket.http')
http.TIMEOUT = 3

local BASE_URL = "http://emoncms.org/input/post.json?apikey=" .. API_KEY
local Log = function (text) luup.log('EnergyMonitor Logger: ' .. (text or "empty")) end
local lastFullUpload = 0

local items = {} -- contains items: { time, deviceId, value }

local function StartCallbackTimer(interval)
	luup.call_delay("EnergyMonitorOnTimer", interval or updateInterval, nil)
end

local function InitWatch()
	StartCallbackTimer(1)
end

local function AddKeyValuePair(key, value)
	local item = string.format("%s:%s", key, tostring(value))
	items[#items + 1] = item
end

local function SerializeData()
	local dataText = "{" .. table.concat(items, ",") .. "}"
	return dataText
end

local function ResetData()
	items = {}
end

local function SendData()
	local data = SerializeData()
	ResetData()
	
	local parameters = "&node=" .. tostring(NODE_ID) .. "&json=" .. data
	local url = BASE_URL .. parameters
	if (DEBUG) then Log("Updating with: " .. parameters) end
	http.request(url)
end

local function AddAllVariablesAndTotal()
	local total = 0
	for i, v in ipairs(VARIABLES) do
		local value
		if v.deviceId then
			value = luup.variable_get(v.serviceId, v.serviceVar, v.deviceId)
		elseif v.calculate then
			value = v.calculate()
		end
		value = tonumber(value) or 0
		if v.serviceVar == "Watts" then
			total = total + value
		end
		AddKeyValuePair(v.key, value)
	end
	AddKeyValuePair(TOTAL_KEY, total)
end

function EnergyMonitorOnTimer()
	StartCallbackTimer()
	AddAllVariablesAndTotal()
	SendData()
end

InitWatch()
